@@ -66,6 +66,7 @@ kucoin_signature <- function(api_secret, time, method, path, body) {
 #' API console.
 #' @param api_secret your Kucoin API secret.
 #' @param query a named list containing your query parameters.
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns the data from your Kucoin API call.
 #' @export
@@ -94,7 +95,8 @@ kucoin_api_call <- function(url
                             , passphrase
                             , version
                             , api_secret
-                            , query = NULL){
+                            , query = NULL
+                            , timeout_seconds = 60){
 
   if (version == '2') {
     api_secret <- stringi::stri_enc_toutf8(api_secret)
@@ -107,19 +109,32 @@ kucoin_api_call <- function(url
     passphrase <- openssl::base64_encode(passphrase)
   }
 
-  res <- httr::VERB(method
-                    , url
-                    , httr::accept("application/json")
-                    , httr::add_headers('KC-API-KEY' = api_key
-                                        , 'KC-API-SIGN' = sig
-                                        , 'KC-API-TIMESTAMP' = time
-                                        , 'KC-API-PASSPHRASE' = passphrase
-                                        , 'KC-API-KEY-VERSION' = version
-                    )
-                    , query = query
-  )
-  data <- jsonlite::fromJSON(rawToChar(res$content))
-  return(data)
+  tryCatch({
+    res <- httr::VERB(method
+                      , url
+                      , httr::accept("application/json")
+                      , httr::add_headers('KC-API-KEY' = api_key
+                                          , 'KC-API-SIGN' = sig
+                                          , 'KC-API-TIMESTAMP' = time
+                                          , 'KC-API-PASSPHRASE' = passphrase
+                                          , 'KC-API-KEY-VERSION' = version
+                      )
+                      , httr::timeout(timeout_seconds)
+                      , query = query
+    )
+
+    if (res$status_code == 200) {
+      data <- jsonlite::fromJSON(rawToChar(res$content))
+      return(data)
+    } else {
+      stop(paste("API call failed with status code", res$status_code))
+    }
+
+  }, error = function(e) {
+    message <- paste("Error during API call:", e$message)
+    warning(message)
+    return(NULL)
+  })
 }
 
 #' kucoin_subaccounts
@@ -130,6 +145,7 @@ kucoin_api_call <- function(url
 #' key.
 #' @param version your API key version. This can be retrieved from your Kucoin
 #' API console. The default value is "2".
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a list containing your Kucoin sub-accounts.
 #' @export
@@ -141,7 +157,7 @@ kucoin_api_call <- function(url
 #' passphrase <- "..."
 #' accounts <- kucoin_subaccounts(api_key, api_secret, passphrase)}
 
-kucoin_subaccounts <- function(api_key, api_secret, passphrase, version = "2"){
+kucoin_subaccounts <- function(api_key, api_secret, passphrase, version = "2", timeout_seconds = 60){
   url <- "https://api.kucoin.com/api/v1/sub/user"
   time <- kucoin_time()
   method <- "GET"
@@ -156,9 +172,20 @@ kucoin_subaccounts <- function(api_key, api_secret, passphrase, version = "2"){
                           , time
                           , passphrase
                           , version
-                          , api_secret)
+                          , api_secret
+                          , timeout_seconds = timeout_seconds)
 
-  return(data$data)
+  if (is.null(data)) {
+    warning("Failed to retrieve data from Kucoin API.")
+    return(NULL)
+  }
+
+  if (!is.null(data$data)) {
+    return(data$data)
+  } else {
+    warning("The response does not contain 'data'.")
+    return(NULL)
+  }
 }
 
 #' kucoin_accounts
@@ -169,6 +196,7 @@ kucoin_subaccounts <- function(api_key, api_secret, passphrase, version = "2"){
 #' key.
 #' @param version your API key version. This can be retrieved from your Kucoin
 #' API console. The default value is "2".
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a dataframe containing your Kucoin accounts and balances.
 #' @export
@@ -183,7 +211,8 @@ kucoin_subaccounts <- function(api_key, api_secret, passphrase, version = "2"){
 kucoin_accounts <- function(api_key
                             , api_secret
                             , passphrase
-                            , version = '2'){
+                            , version = '2'
+                            , timeout_seconds = 60){
   url <- 'https://api.kucoin.com/api/v1/accounts'
   time <- kucoin_time()
   method <- 'GET'
@@ -198,29 +227,62 @@ kucoin_accounts <- function(api_key
                           , time
                           , passphrase
                           , version
-                          , api_secret)
+                          , api_secret
+                          , timeout_seconds = timeout_seconds)
 
-  return(data$data)
+  if (is.null(data)) {
+    warning("Failed to retrieve data from Kucoin API.")
+    return(NULL)
+  }
+
+  if (!is.null(data$data)) {
+    return(data$data)
+  } else {
+    warning("The response does not contain 'data'.")
+    return(NULL)
+  }
 }
 
 #' kucoin_symbols_list
 #'
 #' @param market optionally provide a market to filter on. This function will
 #' return all markets by default.
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a dataframe containing information about trading symbols
 #' @export
 #'
 #' @examples
-#' kucoin_symbols_list('btc')
+#' kucoin_symbols_list('btc', 4.5)
 
-kucoin_symbols_list <- function(market = NULL){
+kucoin_symbols_list <- function(market = NULL, timeout_seconds = 60){
   query <- list(market = toupper(market))
-  res <- httr::VERB('GET'
-                    , 'https://api.kucoin.com/api/v2/symbols'
-                    , httr::accept("application/json")
-                    , query = query
-  )
-  data <- jsonlite::fromJSON(rawToChar(res$content))
-  return(data$data)
+
+  tryCatch({
+    res <- httr::VERB('GET'
+                      , 'https://api.kucoin.com/api/v2/symbols'
+                      , httr::accept("application/json")
+                      , httr::timeout(timeout_seconds)
+                      , query = query
+    )
+
+    if (res$status_code == 200) {
+      data <- jsonlite::fromJSON(rawToChar(res$content))
+
+      if (!is.null(data$data)) {
+        return(data$data)
+      } else {
+        warning("The response does not contain 'data'.")
+        return(NULL)
+      }
+
+    } else {
+      stop(paste("API call failed with status code", res$status_code))
+    }
+
+  }, error = function(e) {
+    message <- paste("Error during API call:", e$message)
+    warning(message)
+    return(NULL)
+  })
 }
