@@ -6,19 +6,21 @@
 #' @param offset optionally specify how many results to skip. This is a numeric
 #' parameter.
 #' @param contract_address optionally filter results by contract address.
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a dataframe containing information about the nifties owned by
 #' the specified user
 #' @export
 #'
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' nifties <- nifty_gateway_user_nifties('tommy')}
 
 nifty_gateway_user_nifties <- function(username
                                        , limit = NULL
                                        , offset = NULL
-                                       , contract_address = NULL) {
+                                       , contract_address = NULL
+                                       , timeout_seconds = 60) {
   url <- paste('https://api.niftygateway.com/v1/users/'
                , username
                , '/nifties/'
@@ -28,10 +30,31 @@ nifty_gateway_user_nifties <- function(username
                 , limit = limit
                 , offset = offset
                 )
-  res <- httr::GET(url
-                  , query = query)
-  data <- jsonlite::fromJSON(rawToChar(res$content))
-  return(data$results)
+
+  tryCatch({
+    res <- httr::GET(url
+                     , query = query
+                     , httr::timeout(timeout_seconds))
+
+    if (res$status_code == 200) {
+      data <- jsonlite::fromJSON(rawToChar(res$content))
+
+      if (!is.null(data$results)) {
+        return(data$results)
+      } else {
+        warning("The response does not contain 'results'.")
+        return(NULL)
+      }
+
+    } else {
+      stop(paste("API call failed with status code", res$status_code))
+    }
+
+  }, error = function(e) {
+    message <- paste("Error during API call:", e$message)
+    warning(message)
+    return(NULL)
+  })
 }
 
 #' nifty_gateway_creators
@@ -41,22 +64,44 @@ nifty_gateway_user_nifties <- function(username
 #' is a numeric parameter.
 #' @param offset optionally specify how many results to skip. This is a numeric
 #' parameter.
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a dataframe containing ownership information for all nifties
 #' created by the given creator and currently stored on Nifty Gateway
 #' @export
 #'
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' creators <- nifty_gateway_creators('beeple')}
 
-nifty_gateway_creators <- function(username, limit = NULL, offset = NULL) {
+nifty_gateway_creators <- function(username, limit = NULL, offset = NULL, timeout_seconds = 60) {
   url <- paste('https://api.niftygateway.com/v1/creators/'
                , username
                , '/collectors/'
                , sep = '')
   query <- list(limit = limit, offset = offset)
-  res <- httr::GET(url, query = query)
-  data <- jsonlite::fromJSON(rawToChar(res$content))
-  return(data$results)
+  #res <- httr::VERB('GET', url, httr::accept("application/json"), query = query)
+
+  tryCatch({
+    res <- httr::GET(url, query = query, httr::timeout(timeout_seconds))
+
+    if (res$status_code == 200) {
+      data <- jsonlite::fromJSON(rawToChar(res$content))
+
+      if (!is.null(data$results)) {
+        return(data$results)
+      } else {
+        warning("The response does not contain 'results'.")
+        return(NULL)
+      }
+
+    } else {
+      stop(paste("API call failed with status code", res$status_code))
+    }
+
+  }, error = function(e) {
+    message <- paste("Error during API call:", e$message)
+    warning(message)
+    return(NULL)
+  })
 }
