@@ -1,16 +1,36 @@
 #' okcoin_trading_pairs
 #'
+#' @param timeout_seconds seconds until the query times out. Default is 60.
+#'
 #' @return returns a dataframe containing information about all trading pairs on
 #' Okcoin
 #' @export
 #'
 #' @examples
-#' okcoin_trading_pairs()
+#' okcoin_trading_pairs(4.5)
 
-okcoin_trading_pairs <- function() {
-  res <- httr::GET("https://www.okcoin.com/api/v5/public/instruments?instType=SPOT")
-  data <- jsonlite::fromJSON(rawToChar(res$content))
-  return(data$data)
+okcoin_trading_pairs <- function(timeout_seconds = 60) {
+  tryCatch({
+    res <- httr::GET("https://www.okcoin.com/api/v5/public/instruments?instType=SPOT"
+                     , httr::timeout(timeout_seconds))
+
+    if (res$status_code == 200) {
+      data <- jsonlite::fromJSON(rawToChar(res$content))
+      if (!is.null(data$data)) {
+        return(data$data)
+      } else {
+        warning("The response does not contain 'data'.")
+        return(NULL)
+      }
+    } else {
+      stop(paste("API call failed with status code", res$status_code))
+    }
+
+  }, error = function(e) {
+    message <- paste("Error during API call:", e$message)
+    warning(message)
+    return(NULL)
+  })
 }
 
 #' okcoin_orders
@@ -23,6 +43,7 @@ okcoin_trading_pairs <- function() {
 #' @param state Order Status: -1: Canceled, 0: Open, 1: Partially Filled,
 #' 2: Fully Filled, 3: Submitting, 4: Canceling, 6: Incomplete (open + partially
 #' filled), 7: Complete (canceled + fully filled)
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a dataframe containing your orders from the most recent 3
 #' months
@@ -37,13 +58,20 @@ okcoin_trading_pairs <- function() {
 #' state <- '2'
 #' orders <- okcoin_orders(secret, key, passphrase, instrument_id, state)}
 
-okcoin_orders <- function(secret, key, passphrase, instrument_id, state) {
+okcoin_orders <- function(secret, key, passphrase, instrument_id, state, timeout_seconds = 60) {
   path <- paste('/api/spot/v3/orders?instrument_id=', instrument_id, '&state=', state, sep = '')
   url <- paste('https://www.okcoin.com', path, sep = '')
   formatted_time <- okcoin_time()
   signature <- okcoin_signature(path, secret, formatted_time, 'GET')
-  data <- okcoin_api_call(url, key, signature, formatted_time, passphrase)
-  return(data)
+  data <- okcoin_api_call(url, key, signature, formatted_time, passphrase, timeout_seconds)
+
+  if (is.null(data)) {
+    warning("Failed to retrieve data from OkCoin API.")
+    return(NULL)
+  } else {
+    return(data)
+  }
+
 }
 
 #' okcoin_spot_account_info
@@ -52,6 +80,7 @@ okcoin_orders <- function(secret, key, passphrase, instrument_id, state) {
 #' @param key your API key for Okcoin
 #' @param passphrase the passphrase which you created when generating your
 #' Okcoin API key
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a dataframe containing your spot account balances
 #' @export
@@ -63,13 +92,20 @@ okcoin_orders <- function(secret, key, passphrase, instrument_id, state) {
 #' passphrase <- "..."
 #' balances <- okcoin_spot_account_info(secret, key, passphrase)}
 
-okcoin_spot_account_info <- function(secret, key, passphrase) {
+okcoin_spot_account_info <- function(secret, key, passphrase, timeout_seconds = 60) {
   path <- paste('/api/spot/v3/accounts', sep = '')
   url <- paste('https://www.okcoin.com', path, sep = '')
   formatted_time <- okcoin_time()
   signature <- okcoin_signature(path, secret, formatted_time, 'GET')
-  data <- okcoin_api_call(url, key, signature, formatted_time, passphrase)
-  return(data)
+  data <- okcoin_api_call(url, key, signature, formatted_time, passphrase, timeout_seconds)
+
+  if (is.null(data)) {
+    warning("Failed to retrieve data from OkCoin API.")
+    return(NULL)
+  } else {
+    return(data)
+  }
+
 }
 
 #' okcoin_time
@@ -126,6 +162,7 @@ okcoin_signature <- function(path, secret, formatted_time, method) {
 #' format
 #' @param passphrase the passphrase which you created when generating your
 #' Okcoin API key
+#' @param timeout_seconds seconds until the query times out. Default is 60.
 #'
 #' @return returns a dataframe containing the results of your API call
 #' @export
@@ -142,14 +179,27 @@ okcoin_signature <- function(path, secret, formatted_time, method) {
 #' passphrase <- "..."
 #' data <- okcoin_api_call()}
 
-okcoin_api_call <- function(url, key, signature, formatted_time, passphrase) {
-  res <- httr::GET(url,
-              httr::add_headers('Content-Type' = 'application/json'
-                          , 'OK-ACCESS-KEY' = key
-                          , 'OK-ACCESS-SIGN' = signature
-                          , 'OK-ACCESS-TIMESTAMP' = formatted_time
-                          , 'OK-ACCESS-PASSPHRASE' = passphrase
-              ))
-  data <- jsonlite::fromJSON(rawToChar(res$content))
-  return(data)
+okcoin_api_call <- function(url, key, signature, formatted_time, passphrase, timeout_seconds = 60) {
+  tryCatch({
+    res <- httr::GET(url,
+                     httr::add_headers('Content-Type' = 'application/json'
+                                       , 'OK-ACCESS-KEY' = key
+                                       , 'OK-ACCESS-SIGN' = signature
+                                       , 'OK-ACCESS-TIMESTAMP' = formatted_time
+                                       , 'OK-ACCESS-PASSPHRASE' = passphrase
+                     )
+                     , httr::timeout(timeout_seconds))
+
+    if (res$status_code == 200) {
+      data <- jsonlite::fromJSON(rawToChar(res$content))
+      return(data)
+    } else {
+      stop(paste("API call failed with status code", res$status_code))
+    }
+
+  }, error = function(e) {
+    message <- paste("Error during API call:", e$message)
+    warning(message)
+    return(NULL)
+  })
 }
