@@ -1,9 +1,11 @@
 test_that('the "coinbase_all_currencies" function returns a dataframe', {
+  skip_on_cran()
   expect_s3_class(coinbase_all_currencies(), 'data.frame')
 })
 
 test_that('the "coinbase_single_currency" function returns a list whether a
           currency is supplied and regardless of symbol casing', {
+            skip_on_cran()
   expect_type(coinbase_single_currency(''), 'list')
   expect_type(coinbase_single_currency('btc'), 'list')
   expect_type(coinbase_single_currency('bTc'), 'list')
@@ -12,9 +14,11 @@ test_that('the "coinbase_single_currency" function returns a list whether a
 
 test_that('the "coinbase_single_currency" function will return a "NotFound"
           message when applicable', {
-  expect_true(coinbase_single_currency('MADEUPASSET')$message == "NotFound")
+            skip_on_cran()
+  expect_true(is.null(coinbase_single_currency('MADEUPASSET')))
 })
 
 test_that('the "coinbase_time" function returns a double', {
+  skip_on_cran()
   expect_type(coinbase_time(), 'double')
 })
